=begin rdoc
= Greeble_clf.rb
Copyright 2009, Author
All Rights Reserved

== Disclaimer
THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES,
INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
FOR A PARTICULAR PURPOSE.

== License
Permission to use, copy, modify, and distribute this software for noncommercial
purposes and without fee is hereby granted, provided that the above copyright
notice appears in all copies. Commercial distribution, in whole or in part, is
expressly prohibited without the expressed, written consent of the author.

== Information
Author:: Chris Fullmer
Organization:: www.chrisfullmer.com
Name:: clfgreeble.rb
Version:: beta 0.1
SU Version:: I developed this on v7, but old versions should work.
Date:: 2-1-09
Description:: This script adds Greebles to all selected faces or to all loose
geometry if nothing is selected
Usage:: Select the faces you would like to be Greebled.  Then run the script
(Plugins>Greeble).  Put in min/max height for Greebles.  You can also add an
offset to be used. A negative value is an inset face.  The Greeble offset or
original face will change which face gets greebled.  I think that playing with
that option will help explain it.
History::
0.01:: 2009-02-01
   * Original release. Plenty of bugs.  Some known, probably lots of unknown.
0.011:: 2009-02-01
   * Upgrade to the original.  Now it supports the SketchUp 7 ruby speedboost.
0.02:: 2009-02-02
   * Added min/max options to the offset. 
   * Added some checks on the menu inputs to help make sure that useable values are entered.
   * Added rough metric support.
0.025:: 2009-02-03
   * Fixed a bunch of errors in previous version including the random # generator, offset distances work consistently.
   * Added a face normal check to make sure new faces created with offset match the original face direction.
0.03:: 2009-02-06
   * Added a validity check on the face about to be push/pulled to reduce breakage.
   * Ability to Set min/max offset distances is now much more stable.
   * Correctly uses the speed up capability of SU 7.
   * Works with SU 6 (and lower).
   * Uses (and requires) ProgressBar.rb to show its working and estimate how much time it will take.
0.04:: 2009-02-XX
   * Added "Happy Greeble-ing" to the status bar
=end

require "sketchup.rb"
require "offset.rb"
require "progressbar.rb"

module Clfgreeble

	def Clfgreeble.greeble
		model = Sketchup.active_model
		clselection = model.selection
		retry_menu = 0
		values = ["1'", "5'", "-1'", "-2'", "0"]
		result = Sketchup.set_status_text( "Happy Greeble-ing! Chris", SB_PROMPT )
		 
		# This checks to see if the user has anything selected.  If the selection is empty, a warning is issued.
		if clselection.empty?
			UI.messagebox "You have nothing selected.  This will Greeble all faces that are not grouped."
			entities = model.active_entities
		else
			entities = model.selection
		end
		 
		# This is the first menu.  It is set to run while the variable retry_menu is set to 0.  So once the menu is complete,
		# the retry_menu is set to 1.  But if there was an incorrect value entered, then retry_menu gets set back to zero and the
		# menu is re-run.
		while retry_menu == 0
			prompts = ["Min Height", "Max Height", "Min Offset Distance (negative is inward)", "Max offset Distance (negative is inward)",
			"Greeble the offset face(0) or original face(1)?"]
			results = inputbox prompts, values, "Box Dimensions"
			retry_menu = 1
			if results[0].to_l > results[1].to_l
				UI.messagebox "The Minimum height needs to be equal to or smaller than the maximum height."
				retry_menu = 0
				results[0] = results[1]
			end
			if results[4].to_i == 0 or results[4].to_i == 1
			else
				UI.messagebox "Greeble the offset(0) or original(1) face can only be set to 0 or 1."
				retry_menu = 0
				results[4] = 0
			end
			if	(results[2].to_l >= 0 and results[3].to_l >= 0) or (results[2].to_l <= 0 and results[3].to_l <= 0)
				if results[3].to_l.abs - results[2].to_l.abs < 0
					retry_menu = 0
					results[2] = results[3]
					UI.messagebox "Max offset must be a larger number than min offset"
				end
			else
				UI.messagebox "Offset Min/Max values need to be either both positive or both negative"
				retry_menu = 0
			end	
			values = [results[0], results[1], results[2], results[3], results[4]]
		end
		
		clent = entities[0]
		clcount = 0
		clface = []
		
		a = Sketchup.version
		b = a[0,1].to_i
		if b > 6
			model.start_operation("Greeble", true)
		else
			model.start_operation("Greeble")
		end
		
		# This goes through all entities puts them one by one to the clent variable.  Then if clent
		# is holding a face, that fave gets appended to the end of the clface array.  This leaves me
		# with an array of just the faces.
		pb = ProgressBar.new(entities.length,"Finding Faces from #{entities.length} total entities")
		entities.each do
			clent = entities[clcount]
			pb.update(clcount)
				if clent.typename == ("Face")
					clface.push clent
					clcount = clcount + 1
				else
					clcount = clcount + 1
				end
		end
		
		clrandheightdif = results[1].to_l - results[0].to_l
		clrandoffsetdif = results[3].to_l - results[2].to_l
		cloffset = clrandoffsetdif + results[2].to_i + results[3].to_i
		clrandoffsetneg = 1        # Arbitrarily sets the value to 1 which is Positive.
		if results[3].to_l < 0     # If the offset values are negative, then it changes the value to 0.
			clrandoffsetneg = 0
		end
		pb2 = ProgressBar.new(clface.length,"Making #{clface.length} Greebles")
		clcount = 0
		clface.each do
			pb2.update(clcount)
			if clface[clcount].valid?
				if results[2].to_i == 0 and results[3].to_i == 0
					if clrandheightdif == 0
						clface[clcount].pushpull results[0].to_l
					else
						clpp = rand(clrandheightdif) + results[0].to_l + 1
						clface[clcount].pushpull clpp, true
					end
					clcount = clcount + 1
				else
					clpp = rand(clrandheightdif) + results[0].to_l + 1
					if clrandoffsetdif == 0
						cl_newface = clface[clcount].offset results[2].to_l
					else
						if clrandoffsetneg == 1 #decides if the randomoffset difference is negative. 0 is negative.
							clrandoffset = rand(clrandoffsetdif) + results[2].to_l + 1
							cl_newface = clface[clcount].offset clrandoffset
						else
							clrandoffset = -(rand(clrandoffsetdif.abs) + results[2].to_l.abs + 1)
							cl_newface = clface[clcount].offset clrandoffset
						end 
					end
					
					# Tests the direction of the new face to the original face.  If they don't face the same direction,
					# the new face gets reversed to match the original face.
					orig_vector = clface[clcount].normal
					new_vector = cl_newface.normal
					if orig_vector != new_vector
						cl_newface.reverse!
					end
					
					if clrandheightdif == 0
						if results[4].to_i == 0
							cl_newface.pushpull results[0].to_l, true
						else
							clface[clcount].pushpull results[0].to_l, true
						end
					else
						if results[4].to_i == 0
							cl_newface.pushpull clpp, true
						else
							clface[clcount].pushpull clpp, true
						end
					end
					clcount = clcount + 1
				end
			else
				puts clface[clcount].valid?.to_s + " " + clface[clcount].to_s
				clcount = clcount + 1
			end
		end
	model.commit_operation
	entities = model.active_entities
	end
end

unless file_loaded?("clf_greeble.rb")
   file_loaded("clf_greeble.rb")
# BEGIN CHANGES BY organizerEdit.rb
$submenu ? (organizer = $submenu) : (organizer = UI.menu("Plugins"))
   organizer.add_item("Greeble Beta") { Clfgreeble.greeble }
# END CHANGES BY organizerEdit.rb
end